@@ -1,6 +1,6 @@
 /* simple-libmpd.c
  *
- * Copyright (c) 2006-2008 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2010 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -37,17 +37,15 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#define STRLENGTH 32
-
 MpdObj* mpd_new(char* host, int port, char* pass)
 {
    MpdObj* mo = g_new0(MpdObj,1);
 
    DBG("host=%s, port=%d, pass=%s", host, port, pass);
 
-   mo->host = g_strndup(host,STRLENGTH);
+   mo->host = g_strdup(host);
    mo->port = port;
-   mo->pass = g_strndup(pass,STRLENGTH);
+   mo->pass = g_strdup(pass);
    mo->socket = 0;
    mo->status = 0;
    mo->repeat = 0;
@@ -436,6 +434,34 @@ void parse_playlistinfo_answer(MpdObj *mo, void *param)
    DBG("Got 'OK', md->nb = %d", md->nb);
 }
 
+void parse_outputs_answer(MpdObj *mo, void *param)
+{
+   MpdData* md = (MpdData*) param;
+   gchar **lines, **tokens;
+   int i = 0;
+   lines = g_strsplit(mo->buffer, "\n", 0);
+   while(lines[i] && strcmp(lines[i],"OK"))
+   {
+      md->alloutputs[md->nb] = g_new(mpd_Output, 1);
+      md->alloutputs[md->nb]->enabled = -1;
+      DBG("Going to parse output #%d", md->nb);
+      while(lines[i] && md->alloutputs[md->nb]->enabled < 0)
+      {
+         tokens = g_strsplit(lines[i], ":", 2);
+         /* remove leading whitespace */
+         tokens[1] = g_strchug(tokens[1]);
+         DBG("key=\"%s\",value=\"%s\"", tokens[0], tokens[1]);
+         if      (0 == strcmp("outputid",tokens[0])) md->alloutputs[md->nb]->id = atoi(tokens[1]);
+         else if (0 == strcmp("outputname",tokens[0])) md->alloutputs[md->nb]->name = g_strdup(tokens[1]);
+         else if (0 == strcmp("outputenabled",tokens[0])) md->alloutputs[md->nb]->enabled = atoi(tokens[1]);
+         i++;
+         g_strfreev(tokens);
+      }
+      md->nb++;
+   }
+   g_strfreev(lines);
+}
+
 mpd_Song* mpd_playlist_get_current_song(MpdObj* mo)
 {
    DBG("!");
@@ -473,6 +499,7 @@ MpdData* mpd_playlist_get_changes(MpdObj* mo, int old_playlist_id)
 {
    MpdData* md = g_new0(MpdData,1);
    md->cur = md->nb = 0;
+   md->type = MPD_DATA_TYPE_SONG;
    md->allsongs = g_new(mpd_Song, mo->playlistlength);
    DBG("!");
    send_complex_cmd(mo, "playlistinfo\n", parse_playlistinfo_answer, (void*) md);
@@ -488,27 +515,69 @@ MpdData* mpd_data_get_next(MpdData* md)
    {
       for (md->cur--; md->cur; md->cur--)
       {
-         if (md->allsongs[md->cur].file) free(md->allsongs[md->cur].file);
-         if (md->allsongs[md->cur].artist) free(md->allsongs[md->cur].artist);
-         if (md->allsongs[md->cur].album) free(md->allsongs[md->cur].album);
-         if (md->allsongs[md->cur].title) free(md->allsongs[md->cur].title);
-         if (md->allsongs[md->cur].track) free(md->allsongs[md->cur].track);
+         switch (md->type) {
+            case MPD_DATA_TYPE_OUTPUT_DEV:
+            if (md->alloutputs[md->cur]->name) free(md->alloutputs[md->cur]->name);
+            break;
+            case MPD_DATA_TYPE_SONG:
+            if (md->allsongs[md->cur].file) free(md->allsongs[md->cur].file);
+            if (md->allsongs[md->cur].artist) free(md->allsongs[md->cur].artist);
+            if (md->allsongs[md->cur].album) free(md->allsongs[md->cur].album);
+            if (md->allsongs[md->cur].title) free(md->allsongs[md->cur].title);
+            if (md->allsongs[md->cur].track) free(md->allsongs[md->cur].track);
+            break;
+         }
+      }
+      switch (md->type) {
+         case MPD_DATA_TYPE_OUTPUT_DEV:
+            g_free(md->alloutputs);
+            break;
+         case MPD_DATA_TYPE_SONG:
+            g_free(md->allsongs);
+            break;
       }
-      g_free(md->allsongs);
       g_free(md);
       DBG("Free()'d md");
       return NULL;
    }
-   md->song = (&md->allsongs[md->cur]);
+   switch (md->type) {
+      case MPD_DATA_TYPE_OUTPUT_DEV:
+         md->output_dev = md->alloutputs[md->cur];
+         break;
+      case MPD_DATA_TYPE_SONG:
+         md->song = (&md->allsongs[md->cur]);
+         break;
+   }
    return md;
 }
 
+MpdData* mpd_server_get_output_devices(MpdObj* mo)
+{
+   MpdData* md = g_new0(MpdData,1);
+   DBG("!");
+   md->cur = md->nb = 0;
+   md->alloutputs = g_new(mpd_Output*,1);
+   md->type = MPD_DATA_TYPE_OUTPUT_DEV;
+   send_complex_cmd(mo, "outputs\n", parse_outputs_answer, (void*) md);
+   md->output_dev = md->alloutputs[0];
+   return ((!mo->error) ? md : NULL);
+}
+
+int mpd_server_set_output_device (MpdObj* mo, int id, int state)
+{
+   char outbuf[18];
+   /* write (enable|disable)output 'id' to socket */
+   DBG("!");
+   snprintf(outbuf, sizeof(outbuf), "%soutput %d\n",(state ? "enable" : "disable"), id);
+   return mpd_send_single_cmd(mo,outbuf);
+}
+
 void mpd_status_set_volume(MpdObj* mo, int newvol)
 {
    char outbuf[15];
    /* write setvol 'newvol' to socket */
    DBG("!");
-   sprintf(outbuf,"setvol %d\n",newvol);
+   snprintf(outbuf, sizeof(outbuf), "setvol %d\n",newvol);
    mpd_send_single_cmd(mo,outbuf);
 }
 
@@ -528,7 +597,7 @@ int mpd_player_set_random(MpdObj* mo, int random)
 {
    char outbuf[15];
    DBG("!");
-   sprintf(outbuf,"random %d\n",random);
+   snprintf(outbuf, sizeof(outbuf), "random %d\n",random);
    return mpd_send_single_cmd(mo,outbuf);
 
 }
@@ -537,7 +606,7 @@ int mpd_player_set_repeat(MpdObj* mo, int repeat)
 {
    char outbuf[15];
    DBG("!");
-   sprintf(outbuf,"repeat %d\n",repeat);
+   snprintf(outbuf, sizeof(outbuf), "repeat %d\n",repeat);
    return mpd_send_single_cmd(mo,outbuf);
 }
 
@@ -584,7 +653,7 @@ int mpd_player_play_id(MpdObj* mo, int id)
 {
    char outbuf[15];
    DBG("!");
-   sprintf(outbuf,"playid %d\n",id);
+   snprintf(outbuf, sizeof(outbuf), "playid %d\n",id);
    return mpd_send_single_cmd(mo,outbuf);
 }
 
@@ -597,9 +666,16 @@ int mpd_check_error(MpdObj* mo)
 void mpd_send_password(MpdObj* mo)
 {
    DBG("!");
-   char outbuf[30];
+   char outbuf[256];
    /* write password 'password' to socket */
-   sprintf(outbuf,"password %s\n",mo->pass);
+   int wrote = snprintf(outbuf, sizeof(outbuf), "password %s\n",mo->pass);
+   if (wrote > 255) {
+	/* the password is too long to fit though there doesn't seem to be a
+	 * nice way to report this error :-/ */
+	fprintf(stderr, "xfce4-mpc-plugin: password too long!\n");
+	mo->error = MPD_ERROR_SYSTEM;
+	return;
+   }
    mpd_send_single_cmd(mo,outbuf);
 }
 
@@ -607,14 +683,14 @@ void mpd_set_hostname(MpdObj* mo, char* host)
 {
    DBG("! new hostname=%s",host);
    g_free(mo->host);
-   mo->host = g_strndup(host,STRLENGTH);
+   mo->host = g_strdup(host);
 }
 
 void mpd_set_password(MpdObj* mo, char* pass)
 {
    DBG("! new password=%s",pass);
    g_free(mo->pass);
-   mo->pass = g_strndup(pass,STRLENGTH);
+   mo->pass = g_strdup(pass);
 }
 
 void mpd_set_port(MpdObj* mo,int port)
