@@ -1,6 +1,6 @@
 /* xfce4-mpc-plugin.c
  *
- * Copyright (c) 2006-2009 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2010 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -347,8 +347,62 @@ mpc_repeat_toggled(GtkWidget *widget, t_mpc* mpc)
    mpd_player_set_repeat(mpc->mo, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)));
 }
 
+static void
+mpc_output_toggled(GtkWidget *widget, t_mpc* mpc)
+{
+   DBG("!");
+   int i;
+   /* lookup menuitem */
+   for (i = 0; i < mpc->nb_outputs && mpc->mpd_outputs[i]->menuitem != widget; i++);
+   if (i != mpc->nb_outputs) /* oops case ? */
+      /* set corresponding mpd output status */
+      mpd_server_set_output_device(mpc->mo, mpc->mpd_outputs[i]->id, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)));
+}
 
-void
+/* get output list from mpd, add a checkbox to rightclick menu for each */
+static void
+mpc_update_outputs(t_mpc* mpc)
+{
+   DBG("!");
+   GtkWidget* menu;
+   int i,j=0, old_nb_outputs = mpc->nb_outputs;
+   MpdData * data = mpd_server_get_output_devices(mpc->mo);
+   do {
+      DBG("got output %d with name %s, enabled=%d",data->output_dev->id,data->output_dev->name,data->output_dev->enabled);
+      /* check if this output doesn't already exist */
+      for (i = 0; i < mpc->nb_outputs && mpc->mpd_outputs[i]->id != data->output_dev->id ; i++);
+
+      if (i == mpc->nb_outputs) {
+         DBG("output not found, adding a new checkitem at pos %d",i);
+         GtkWidget* chkitem = gtk_check_menu_item_new_with_label (data->output_dev->name);
+         g_signal_connect (G_OBJECT(chkitem), "toggled", G_CALLBACK (mpc_output_toggled), mpc);
+         xfce_panel_plugin_menu_insert_item(mpc->plugin,GTK_MENU_ITEM(chkitem));
+         /* XXX HACK */
+         menu = g_object_get_data (G_OBJECT (mpc->plugin), I_("xfce-panel-plugin-menu"));
+         gtk_menu_reorder_child(GTK_MENU(menu),chkitem, 12 + i); /* 12 is after 'Outputs' menuitem */
+         gtk_widget_show (chkitem);
+         mpc->mpd_outputs[i] = g_new(t_mpd_output,1);
+         mpc->mpd_outputs[i]->id = data->output_dev->id;
+         mpc->mpd_outputs[i]->menuitem = chkitem;
+         mpc->nb_outputs++;
+      }
+      mpc->mpd_outputs[i]->enabled = data->output_dev->enabled;
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->mpd_outputs[i]->menuitem), mpc->mpd_outputs[i]->enabled);
+      j++;
+   } while (NULL != (data = mpd_data_get_next (data)));
+   /* something changed, better prune the list and recreate it */
+   if (j != mpc->nb_outputs || (old_nb_outputs && j != old_nb_outputs)) {
+      DBG("didnt found same amount of outputs (was %d got %d), resetting output list", (old_nb_outputs ? old_nb_outputs : mpc->nb_outputs), j);
+      for (i = 0; i < mpc->nb_outputs ; i++) {
+         gtk_widget_destroy(mpc->mpd_outputs[i]->menuitem);
+         g_free(mpc->mpd_outputs[i]);
+      }
+      mpc->nb_outputs = 0;
+      mpc_update_outputs(mpc);
+   }
+}
+
+static void
 str_replace(GString *str, gchar* pattern, gchar* replacement)
 {
    if (!replacement)
@@ -358,7 +412,7 @@ str_replace(GString *str, gchar* pattern, gchar* replacement)
    g_free(nstr);
 }
 
-void
+static void
 format_song_display(mpd_Song* song, GString *str, t_mpc* mpc)
 {
    if (0 == str->len)
@@ -418,6 +472,7 @@ enter_cb(GtkWidget *widget, GdkEventCrossing* event, t_mpc* mpc)
 
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->random), mpd_player_get_random(mpc->mo));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->repeat), mpd_player_get_repeat(mpc->mo));
+   mpc_update_outputs(mpc);
 
    gtk_widget_set_tooltip_text (widget, str->str);
    g_string_free(str, TRUE);
@@ -635,6 +690,21 @@ new_button_with_cbk(XfcePanelPlugin * plugin, GtkWidget *parent, gpointer cb, gp
    return button;
 }
 
+static void
+add_separator_and_label_with_markup(XfcePanelPlugin* plugin, gchar* lbl)
+{
+   GtkWidget *separator, *menuitem, *label;
+   separator = gtk_separator_menu_item_new();
+   menuitem = gtk_menu_item_new_with_label(lbl);
+   gtk_widget_set_sensitive(menuitem, FALSE);
+   label = gtk_bin_get_child(GTK_BIN(menuitem));
+   gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+   xfce_panel_plugin_menu_insert_item(plugin,GTK_MENU_ITEM(separator));
+   xfce_panel_plugin_menu_insert_item(plugin,GTK_MENU_ITEM(menuitem));
+   gtk_widget_show (separator);
+   gtk_widget_show (menuitem);
+}
+
 static t_mpc*
 mpc_create (XfcePanelPlugin * plugin)
 {
@@ -671,9 +741,12 @@ mpc_create (XfcePanelPlugin * plugin)
    g_signal_connect (G_OBJECT(mpc->repeat), "toggled", G_CALLBACK (mpc_repeat_toggled), mpc);
    mpc->appl = gtk_menu_item_new_with_label (_("Launch"));
    g_signal_connect (G_OBJECT(mpc->appl), "activate", G_CALLBACK (mpc_launch_client), mpc);
+
+   add_separator_and_label_with_markup(plugin, _("<b><i>Commands</i></b>"));
    xfce_panel_plugin_menu_insert_item(plugin,GTK_MENU_ITEM(mpc->random));
    xfce_panel_plugin_menu_insert_item(plugin,GTK_MENU_ITEM(mpc->repeat));
    xfce_panel_plugin_menu_insert_item(plugin,GTK_MENU_ITEM(mpc->appl));
+   add_separator_and_label_with_markup(plugin, _("<b><i>Outputs</i></b>"));
    gtk_widget_show (mpc->repeat);
    gtk_widget_show (mpc->random);
    gtk_widget_show (mpc->appl);
@@ -695,7 +768,7 @@ mpc_show_about(XfcePanelPlugin *plugin, t_mpc* mpc)
    }
    ainfo = xfce_about_info_new(_("Xfce4 Mpc Plugin"), PACKAGE_VERSION,
                                _("A simple panel-plugin client for Music Player Daemon"),
-                               _("Copyright (c) 2006-2008 Landry Breuil\n"),
+                               _("Copyright (c) 2006-2010 Landry Breuil\n"),
                                XFCE_LICENSE_BSD);
    xfce_about_info_add_credit(ainfo, "Landry Breuil", "landry@fr.homeunix.org", _("Maintainer, Original Author"));
    xfce_about_info_set_homepage(ainfo, "http://goodies.xfce.org/projects/panel-plugins/xfce4-mpc-plugin");
@@ -736,6 +809,8 @@ mpc_construct (XfcePanelPlugin * plugin)
    mpc->playlist_format = g_strdup("%artist% - %album% -/- (#%track%) %title%");
    mpc->show_frame = TRUE;
    mpc->playlist = NULL;
+   mpc->mpd_outputs = g_new(t_mpd_output*,1);
+   mpc->nb_outputs = 0;
 
    mpc_read_config (plugin, mpc);
 
